@@ -9,17 +9,17 @@ use {
     log::*,
     rand::{thread_rng, Rng},
     solana_core::{sigverify::TransactionSigVerifier, sigverify_stage::SigVerifyStage},
-    solana_perf::{packet::to_packet_batches, packet::PacketBatch, test_tx::test_tx},
+    solana_perf::{
+        packet::{to_packet_batches, PacketBatch},
+        test_tx::test_tx,
+    },
     solana_sdk::{
         hash::Hash,
         signature::{Keypair, Signer},
         system_transaction,
         timing::duration_as_ms,
     },
-    std::{
-        sync::mpsc::channel,
-        time::{Duration, Instant},
-    },
+    std::time::{Duration, Instant},
     test::Bencher,
 };
 
@@ -140,7 +140,7 @@ fn gen_batches(use_same_tx: bool) -> Vec<PacketBatch> {
 fn bench_sigverify_stage(bencher: &mut Bencher) {
     solana_logger::setup();
     trace!("start");
-    let (packet_s, packet_r) = channel();
+    let (packet_s, packet_r) = unbounded();
     let (verified_s, verified_r) = unbounded();
     let verifier = TransactionSigVerifier::default();
     let stage = SigVerifyStage::new(packet_r, verified_s, verifier);
